#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "data.h"
#include "distance.h"



#define NDISTANCE	4
#ifdef VOLT5  //in the 5V system the reference Aref is approx. 4.7V i.o. 3.3V 
              //this reduces the converted voltages by a factor of 0.7
#define GAIN	11957
#define VOFFSET	29
#define ROFFSET	0
#else
#define GAIN	8370
#define VOFFSET	44
#define ROFFSET	0
#endif

typedef struct
{ int gain, voffset, roffset;
} sensordata;

sensordata EEMEM eecurve[NDISTANCE] = {{GAIN, VOFFSET, ROFFSET}, 
                                       {GAIN, VOFFSET, ROFFSET}, 
									   {GAIN, VOFFSET, ROFFSET}, 
									   {GAIN, VOFFSET, ROFFSET}
									  };
#ifdef USE_DISTANCE

sensordata curve[NDISTANCE];
char raw_distance = 0;

static char SumChan;                   /* Number of measured channels */
static char SumCnt;                    /* Number of measured channels */

unsigned AD1_SumV[NDISTANCE];                      /* Temporary sum of measured values */
unsigned AD1_OutV[NDISTANCE];                      /* Sum of measured values */

/* with the current settings (14.7MHz, divide by 128) a conversion (13 cycles) takes approx. 113us.
 * The On_End handler is called
 * after 16 conversions on all four channels, hence every 7.2ms. The effective sample rate therefore
 * is 138Hz. The actual sampling and averaging (integration) takes place at 2215Hz.
 */

void init_distance()
{   eeprom_read_block(curve, eecurve, sizeof(curve));
    SumCnt = 0;                        /* Set the number of conversions */
    SumChan = 0;                       /* Set the number of measured channels to 0 */
/*
    ADMUX = _BV(REFS0);        
	ADCSRA |= _BV(ADEN)|_BV(ADSC)|_BV(ADIF)|ADCCLK;
	while (!(ADCSRA & _BV(ADIF))) ; //busy wait for first conversion
    ADCL; ADCH; //discard first measurement
*/
	ADMUX = _BV(REFS0); //preset the channel for the next measurement
    ADCSRA = _BV(ADSC)|_BV(ADIF)|_BV(ADIE)|DISTANCE_FLAGS;  
}

void store_distance(unsigned char n, int p[3])
{ n--;
  if (n >= NDISTANCE)
    return;
  curve[n].gain = p[0];
  curve[n].voffset = p[1];
  curve[n].roffset = p[2];
  eeprom_write_block_nonblocking((char*)&curve[n], (char*)&eecurve[n], sizeof(sensordata));
}

#if 1
unsigned get_distance(char i)
{ int x = AD1_OutV[i];
  if (raw_distance)
    return x;
  x -= curve[i].voffset;
  if (x <= 0)
    return 1023;
  return curve[i].gain/x + curve[i].roffset;
}
#else
unsigned get_distance(char i)
{ return AD1_OutV[i];
}
#endif


ISR(ADC_vect, ISR_NOBLOCK)
{
  unsigned  tmp;
  tmp = ADCL; //ADCL must be read first!!!
  tmp += ADCH<<8;
  AD1_SumV[SumChan] += tmp;     /* Save measured value */
  if (++SumChan == NDISTANCE) SumChan = 0;
  ADMUX = _BV(REFS0) | SumChan;          /* Start measurement of next channel with minimum variation in delay */
#ifndef AUTOTRIGGER
  ADCSRA |= _BV(ADSC);  //start conversion, alternative is automatic conversions
#endif
  if (SumChan == 0)
  { SumCnt++;
    if ( SumCnt == OVERSAMPLE ) 
	{ int i;
	  for (i = 1; i < NDISTANCE; i++)
	  { AD1_OutV[i-1] = AD1_SumV[i]/OVERSAMPLE;
	    AD1_SumV[i] = 0;
	  }
	  AD1_OutV[NDISTANCE-1] = AD1_SumV[0]/OVERSAMPLE;
      AD1_SumV[0] = 0;
      SumCnt = 0;                   
    }
  }
}

#endif
