#include <avr/io.h>
#include <avr/interrupt.h>
#include "timer.h"
#include "robo_ext.h"
#include "data.h"
#include "rs485.h"
#include "servo.h"
#ifdef USE_STEPPER
#include "stepper.h"
#endif

void init_timer() //set timer to 100us
{ //use timer2 in ctc mode, total divider is approx.73 at low freq and 184 at high freq
  TCCR2A = _BV(WGM21); 
  TCCR2B = _BV(CS21); //prescale by 8
  OCR2A  = F_CPU/8/10000-1;
  TIMSK2 = _BV(OCIE2A); //enable int
}

ISR(TIMER2_COMPA_vect, ISR_NOBLOCK)
{ 
#ifdef USE_ROBO_EXT
  static int ext_timer = 0;
#endif
#ifdef USE_SERVO
  static int servo_timer = 0;
#endif
#ifdef USE_STEPPER
  static int stepper_timer = 0;
//  static char motor = 0;
#endif

//  SETPIN11;

#ifdef USE_SERVO
  servo_timer++;
  if (servo_timer >= 200/NSERVOS) //20ms/servo
  { servo_timer = 0;
    update_servos();
  }
#endif

#ifdef USE_ROBO_EXT
  cli();//ack_timeout is shared
  if (ack_timeout)
  { if (--ack_timeout == 0)
    { sei();
      extension_timeout();
    }
  }
  sei();

  ext_timer++;
  if (ext_timer >= 100) //10ms
  { ext_timer = 0;
    read_extensions(); //start a new cycle of reading extension modules
  }
#endif

#ifdef USE_STEPPER
  stepper_timer++;
  if (stepper_timer >= STEPPER_TIME)
  { stepper_timer = 0;
    update_steppers();
  }
#endif

  cli();//rs485_timeout is shared
  if (rs485_timeout)
  { if (--rs485_timeout == 0)
    { sei();
      timeout_rs485();
    }
  }
  sei();

  if (rs485_delay)
  { if (--rs485_delay == 0)
    {
	  send_rs485();
	}
  }

//  CLRPIN11;
}
